@@ -7,7 +7,7 @@ from .filemanager import uuid_originalname
 from .security.decorators import has_access
 from .widgets import FormWidget, ShowWidget, ListWidget, SearchWidget, ListCarousel
 from .actions import ActionItem
-
+from .models.filters import Filters, FilterRelationOneToMany
 
 def expose(url='/', methods=('GET',)):
     """
@@ -45,8 +45,8 @@ class BaseView(object):
     template_folder = 'templates'
     static_folder='static'
     
-    base_permissions = []
-    actions = []
+    base_permissions = None
+    actions = None
 
     default_view = 'list'
 
@@ -55,12 +55,13 @@ class BaseView(object):
             Initialization of base permissions
             based on exposed methods and actions
         """
-        if not self.base_permissions:
+        if self.base_permissions is None:
             self.base_permissions = [
                 ('can_' + attr_name)
                 for attr_name in dir(self)
                 if hasattr(getattr(self, attr_name),'_urls')
                 ]
+            self.actions = self.actions or []
             for attr_name in dir(self):
                 if hasattr(getattr(self, attr_name),'_action'):
                     action = ActionItem(*attr_name._action, func = getattr(self, attr_name))
@@ -150,59 +151,6 @@ class BaseView(object):
                 return url_for('%s.%s' % (self.baseapp.indexview.endpoint, self.baseapp.indexview.default_view))
             
 
-    def _get_group_by_args(self):
-        group_by = request.args.get('group_by')
-        if not group_by: group_by = ''
-        return group_by
-
-    def _get_page_args(self, pages = {}):
-        """
-            Get page arguments, return a dictionary
-            { <VIEW_NAME>: PAGE_NUMBER }
-        
-            Arguments are passed: page_<VIEW_NAME>=<PAGE_NUMBER>
-        
-        """
-        for arg in request.args:
-            re_match = re.findall('page_(.*)', arg)
-            if re_match:
-                pages[re_match[0]] = int(request.args.get(arg))
-        return pages
-
-
-
-    def _get_order_args(self, orders = {}):
-        """
-            Get order arguments, return a dictionary
-            { <VIEW_NAME>: (ORDER_COL, ORDER_DIRECTION) }
-        
-            Arguments are passed like: _oc_<VIEW_NAME>=<COL_NAME>&_od_<VIEW_NAME>='asc'|'desc'
-        
-        """
-        for arg in request.args:
-            re_match = re.findall('_oc_(.*)', arg)
-            if re_match:
-                orders[re_match[0]] = (request.args.get(arg),request.args.get('_od_' + re_match[0]))
-        return orders
-                
-
-    def _get_filter_args(self, filters={}):
-        for arg in request.args:
-            re_match = re.findall('_flt_(.*)', arg)
-            if re_match:
-                # ignore select2 __None value
-                if request.args.get(arg) not in ('__None',''):
-                    filters[re_match[0]] = request.args.get(arg)
-        return filters
-
-
-    def _get_dict_from_form(self, form, filters={}):
-        for item in form:
-            if item.data:
-                filters[item.name] = item.data
-        return filters
-
-
 
 class IndexView(BaseView):
     """
@@ -233,13 +181,15 @@ class SimpleFormView(BaseView):
     edit_widget = FormWidget
     form_title = ''
     """ The form title to be displayed """
-    form_columns = []
+    form_columns = None
     """ The form columns to include, if empty will include all"""
     form = None
     """ The WTF form to render """
-    form_fieldsets = []
+    form_fieldsets = None
     
     def _init_vars(self):
+        self.form_columns = self.form_columns or []
+        self.form_fieldsets = self.form_fieldsets or []
         list_cols = [field.name for field in self.form.refresh()]
         if self.form_fieldsets:
             self.form_columns = []
@@ -266,7 +216,7 @@ class SimpleFormView(BaseView):
 
     def form_get(self, form):
         """
-        Override this method to implement your form processing
+            Override this method to implement your form processing
         """
         pass
 
@@ -290,7 +240,7 @@ class SimpleFormView(BaseView):
 
     def form_post(self, form):
         """
-        Override this method to implement your form processing
+            Override this method to implement your form processing
         """
         pass
 
@@ -305,18 +255,146 @@ class SimpleFormView(BaseView):
 
 
 
-class BaseCRUDView(BaseView):
+class BaseModelView(BaseView):
+    """
+        The base class of GeneralView and ChartView, all properties are inherited
+        Customize GeneralView and ChartView overriding this properties
+    """
+
+    datamodel = None
+    """ Your sqla model you must initialize it like datamodel = SQLAModel(Permission, session) """
+    search_columns = None
+    """ List with allowed search columns, if not provided all possible search columns will be used """
+    label_columns = None
+    """ 
+        Dictionary of labels for your columns, override this if you want diferent pretify labels 
+        
+        example (will just override the label for name column)::
+        
+            class MyView(GeneralView):
+                datamodel = SQLAModel(MyTable, db.session)
+                label_columns = {'name':'My Name Label Overrride'}
+        
+    """    
+    search_form = None
+    """ To implement your own add WTF form for Search """
+    base_filters = None
+    """ 
+        Filter the view use: ['column_name',BaseFilter,'value'] 
+    
+        example::
+        
+            def get_user():
+                return g.user
+        
+            class MyView(GeneralView):
+                datamodel = SQLAModel(MyTable, db.session)
+                base_filters = [['created_by', FilterEqualFunction, get_user],
+                                ['name', FilterStartsWith, 'a']]
+                                            
+    """
+    _base_filters = None
+    """ Internal base Filter from class Filters will always filter view """
+    _filters = None
+    """ Filters object will calculate all possible filter types based on search_columns """
+
+    def __init__(self, **kwargs):
+        """
+            Constructor
+        """
+        self._base_model_init_vars()
+        self._base_model_init_forms()
+        self._filters = Filters(self.search_columns, self.datamodel)
+        super(BaseModelView, self).__init__(**kwargs)
+        
+
+    def _base_model_init_vars(self):
+        self.label_columns = self.label_columns or {}
+        self.base_filters = self.base_filters or []
+        self._base_filters = Filters().add_filter_list(self.datamodel, self.base_filters)
+        list_cols = self.datamodel.get_columns_list()
+        self.search_columns = self.search_columns or list_cols
+        for col in list_cols:
+            if not self.label_columns.get(col):
+                self.label_columns[col] = self._prettify_column(col)
+        
+    def _base_model_init_forms(self):
+        conv = GeneralModelConverter(self.datamodel)
+        if not self.search_form:
+            self.search_form = conv.create_form(self.label_columns,
+                    {}, {}, [], self.search_columns)
+        
+
+    def _get_group_by_args(self):
+        """
+            Get page arguments for group by
+        """
+        group_by = request.args.get('group_by')
+        if not group_by: group_by = ''
+        return group_by
+
+    def _get_page_args(self):
+        """
+            Get page arguments, returns a dictionary
+            { <VIEW_NAME>: PAGE_NUMBER }
+        
+            Arguments are passed: page_<VIEW_NAME>=<PAGE_NUMBER>
+        
+        """
+        pages = {}
+        for arg in request.args:
+            re_match = re.findall('page_(.*)', arg)
+            if re_match:
+                pages[re_match[0]] = int(request.args.get(arg))
+        return pages
+
+
+    def _get_order_args(self):
+        """
+            Get order arguments, return a dictionary
+            { <VIEW_NAME>: (ORDER_COL, ORDER_DIRECTION) }
+        
+            Arguments are passed like: _oc_<VIEW_NAME>=<COL_NAME>&_od_<VIEW_NAME>='asc'|'desc'
+        
+        """
+        orders = {}
+        for arg in request.args:
+            re_match = re.findall('_oc_(.*)', arg)
+            if re_match:
+                orders[re_match[0]] = (request.args.get(arg),request.args.get('_od_' + re_match[0]))
+        return orders
+
+    def _get_filter_args(self):
+        self._filters.clear_filters()
+        for arg in request.args:
+            re_match = re.findall('_flt_(\d)_(.*)', arg)
+            if re_match:
+                self._filters.add_filter_index(re_match[0][1], int(re_match[0][0]), request.args.get(arg))
+
+
+    def _get_search_widget(self, form = None, exclude_cols = [], widgets = {}):
+        widgets['search'] = self.search_widget(route_base = self.route_base,
+                                                form = form,
+                                                include_cols = self.search_columns,
+                                                exclude_cols = exclude_cols,
+                                                filters = self._filters
+                                                )
+        return widgets
+
+
+
+class BaseCRUDView(BaseModelView):
     """
         The base class of GeneralView, all properties are inherited
         Customize GeneralView overriding this properties
     """
     
-    
-    datamodel = None
-    """ Your sqla model you must initialize it like datamodel = SQLAModel(Permission, session) """
-    related_views = []
-    """ Views that will be displayed related with this one, must be instantiated """
-    
+
+    related_views = None
+    """ 
+        List with instantiated GeneralView classes
+        That will be displayed related with this one using relationship sqlalchemy property
+    """
     list_title = ""
     """ List Title """
     show_title = ""
@@ -326,48 +404,59 @@ class BaseCRUDView(BaseView):
     edit_title = ""
     """ Edit Title """
 
-    
-    list_columns = []
+    list_columns = None
     """ Include Columns for lists view """
-    show_columns = []
+    show_columns = None
     """ Include Columns for show view """
-    add_columns = []
+    add_columns = None
     """ Include Columns for add view """
-    edit_columns = []
+    edit_columns = None
     """ Include Columns for edit view """
-    order_columns = []
+    order_columns = None
     """ Allowed order columns """
-    search_columns = []
-    """ Allowed search columns """
-
-    label_columns = {}
-    description_columns = {}
     
+    description_columns = {}
+    """ description for columns that will be shown on the forms """
     add_form_extra_fields = {}
     """ Add extra fields to the Add form using this property """
     edit_form_extra_fields = {}
     """ Add extra fields to the Edit form using this property """
     
     page_size = 30
-
+    """ Page size """
     
     show_fieldsets = []
-    """ show fieldsets [(<'TITLE'|None>, {'fields':[<F1>,<F2>,...]}),....] """
+    """ 
+        show fieldsets django style [(<'TITLE'|None>, {'fields':[<F1>,<F2>,...]}),....]
+        
+        ::
+        
+            class MyView(GeneralView):
+                datamodel = SQLAModel(MyTable, db.session)
+
+                show_fieldsets = [
+                    ('Summary',{'fields':['name','address','group']}),
+                    ('Personal Info',{'fields':['birthday','personal_phone'],'expanded':False}),
+                    ]
+
+    """
     add_fieldsets = []
-    """ add fieldsets [(<'TITLE'|None>, {'fields':[<F1>,<F2>,...]}),....] """
+    """ 
+        add fieldsets django style (look at show_fieldsets)
+    """
     edit_fieldsets = []
-    """ edit fieldsets [(<'TITLE'|None>, {'fields':[<F1>,<F2>,...]}),....] """
+    """ 
+        edit fieldsets django style (look at show_fieldsets)
+    """
 
     
     add_form = None
     """ To implement your own add WTF form for Add """
     edit_form = None
     """ To implement your own add WTF form for Edit """
-    search_form = None
-    """ To implement your own add WTF form for Search """
-
+    
     validators_columns = {}
-
+    """ Add your own validators for forms """
     
     list_template = 'appbuilder/general/model/list.html'
     """ Your own add jinja2 template for list """
@@ -378,7 +467,6 @@ class BaseCRUDView(BaseView):
     show_template = 'appbuilder/general/model/show.html'
     """ Your own add jinja2 template for show """
 
-    
     list_widget = ListWidget
     """ List widget override """
     edit_widget = FormWidget
@@ -388,13 +476,18 @@ class BaseCRUDView(BaseView):
     show_widget = ShowWidget
     """ Show widget override """
     search_widget = SearchWidget
-    """ Search widget override """
+    """ Search widget you can override with your own """
         
     show_additional_links = []
 
+    def __init__(self, **kwargs):
+        super(BaseCRUDView, self).__init__(**kwargs)
+        self._init_titles()
+        self._init_vars()
+        self._init_forms()
+
     
     def _init_forms(self):
-        
         conv = GeneralModelConverter(self.datamodel)        
         if not self.add_form:
             self.add_form = conv.create_form(self.label_columns,
@@ -408,12 +501,7 @@ class BaseCRUDView(BaseView):
                     self.validators_columns,
                     self.edit_form_extra_fields,
                     self.edit_columns)
-        if not self.search_form:
-            self.search_form = conv.create_form(self.label_columns,
-                    self.description_columns,
-                    self.validators_columns,
-                    [],
-                    self.search_columns)
+        
 
     def _init_titles(self):
         if not self.list_title:
@@ -426,12 +514,11 @@ class BaseCRUDView(BaseView):
             self.show_title = 'Show ' + self._prettify_name(self.datamodel.obj.__name__)
 
     def _init_vars(self):
-        self._init_titles()
-            
+        self.related_views = self.related_views or []
+        order_cols = self.datamodel.get_order_columns_list()
         list_cols = self.datamodel.get_columns_list()
-        for col in list_cols:
-            if not self.label_columns.get(col):
-                self.label_columns[col] = self._prettify_column(col)
+        self.list_columns = self.list_columns or [order_cols[0]]
+        self.order_columns = self.order_columns or order_cols
         if self.show_fieldsets:
             self.show_columns = []
             for fieldset_item in self.show_fieldsets:                
@@ -453,42 +540,40 @@ class BaseCRUDView(BaseView):
         else:
             if not self.edit_columns:
                 self.edit_columns = list_cols
-                        
         
-    def __init__(self, **kwargs):
-        self._init_vars()
-        self._init_forms()
-        super(BaseCRUDView, self).__init__(**kwargs)
-    
 
     def _get_related_list_widget(self, item, related_view, 
-                                filters={}, order_column='', order_direction='',
+                                order_column='', order_direction='',
                                 page=None, page_size=None):
 
         fk = related_view.datamodel.get_related_fk(self.datamodel.obj)
-        filters[fk] = item
-        return related_view._get_list_widget(filters = filters, 
-                    order_column = order_column, order_direction = order_direction, page=page, page_size=page_size)
-        
-    def _get_related_list_widgets(self, item, filters = {}, orders = {}, 
-                                pages=None, widgets = {}, **args):
+        filters = Filters().add_filter(fk, FilterRelationOneToMany, 
+                related_view.datamodel, self.datamodel.get_pk_value(item))
+        return related_view._get_list_widget(filters = filters,
+                    order_column = order_column,
+                    order_direction = order_direction,
+                    page=page, page_size=page_size)
+
+    def _get_related_list_widgets(self, item, orders = {}, pages=None, widgets = {}, **args):
         widgets['related_lists'] = []
         for view in self.related_views:
             if orders.get(view.__class__.__name__):
                 order_column, order_direction = orders.get(view.__class__.__name__)
             else: order_column, order_direction = '',''
-            widgets['related_lists'].append(self._get_related_list_widget(item, view, filters, 
+            widgets['related_lists'].append(self._get_related_list_widget(item, view, 
                     order_column, order_direction, page=pages.get(view.__class__.__name__), page_size=view.page_size).get('list'))
         return widgets
     
-    def _get_list_widget(self, filters = {}, 
+    def _get_list_widget(self, filters, 
                         order_column = '', 
                         order_direction = '',
                         page = None,
                         page_size = None,
                         widgets = {}, **args):
 
-        count, lst = self.datamodel.query(filters, order_column, order_direction, page=page, page_size=page_size)
+        """ get joined base filter and current active filter for query """
+        joined_filters = filters.get_joined_filters(self._base_filters)
+        count, lst = self.datamodel.query(joined_filters, order_column, order_direction, page=page, page_size=page_size)
         pks = self.datamodel.get_keys(lst)
         widgets['list'] = self.list_widget(route_base = self.route_base,
                                                 label_columns = self.label_columns,
@@ -504,14 +589,6 @@ class BaseCRUDView(BaseView):
                                                 )
         return widgets
 
-    def _get_search_widget(self, form = None, exclude_cols = [], widgets = {}):
-        widgets['search'] = self.search_widget(route_base = self.route_base,
-                                                form = form,
-                                                include_cols = self.search_columns,
-                                                exclude_cols = exclude_cols,
-                                                )
-        return widgets
-
 
     def _get_show_widget(self, id, widgets = {}, show_additional_links = []):
         if show_additional_links:
@@ -606,12 +683,12 @@ class GeneralView(BaseCRUDView):
     def __init__(self, **kwargs):
         super(GeneralView, self).__init__(**kwargs)
 
+
     """
     --------------------------------
             LIST
     --------------------------------
     """
-    
     @expose('/list/')
     @has_access
     def list(self):
@@ -623,22 +700,9 @@ class GeneralView(BaseCRUDView):
         else: order_column, order_direction = '',''
         page = self._get_page_args().get(self.__class__.__name__)
 
-        filters = {}
-        filters = self._get_filter_args(filters)
-        if (filters != {}):
-            item = self.datamodel.obj()
-            for filter_key in filters:
-                # on related models translate id to model obj
-                try:
-                    rel_obj = self.datamodel.get_related_obj(filter_key, filters.get(filter_key))
-                    setattr(item, filter_key, rel_obj)
-                    form = self.search_form(obj = item)
-                    filters[filter_key] = rel_obj
-                except:
-                    setattr(item, filter_key, filters.get(filter_key))
-                    form = self.search_form(obj = item)
-
-        widgets = self._get_list_widget(filters = filters, 
+        self._get_filter_args()
+        
+        widgets = self._get_list_widget(filters = self._filters, 
                     order_column = order_column, 
                     order_direction = order_direction, 
                     page = page, 
@@ -666,8 +730,9 @@ class GeneralView(BaseCRUDView):
         pages = self._get_page_args()
         orders = self._get_order_args()
         
-        widgets = self._get_related_list_widgets(item, filters = {}, orders = orders, 
+        widgets = self._get_related_list_widgets(item, orders = orders, 
                 pages = pages, widgets = widgets)
+        
         return render_template(self.show_template,
                            pk = pk,
                            title = self.show_title,
@@ -676,7 +741,6 @@ class GeneralView(BaseCRUDView):
                            related_views = self.related_views)
 
 
-
     """
     ---------------------------
             ADD
@@ -686,18 +750,19 @@ class GeneralView(BaseCRUDView):
     @has_access
     def add(self):
 
-        filters = self._get_filter_args(filters={})
+        self._get_filter_args()
 
         form = self.add_form.refresh()
-        exclude_cols = self.datamodel.get_relation_filters(filters)
+        exclude_cols = self._filters.get_relation_cols()
 
         if form.validate_on_submit():
             item = self.datamodel.obj()
             form.populate_obj(item)
+            print "EXT", exclude_cols
             for filter_key in exclude_cols:
-                rel_obj = self.datamodel.get_related_obj(filter_key, filters.get(filter_key))
+                rel_obj = self.datamodel.get_related_obj(filter_key, self._filters.get_filter_value(filter_key))
                 setattr(item, filter_key, rel_obj)
-                        
+
             self.pre_add(item)
             self.datamodel.add(item)
             self.post_add(item)
@@ -722,8 +787,8 @@ class GeneralView(BaseCRUDView):
         orders = self._get_order_args()
         
         item = self.datamodel.get(pk)
-        filters = self._get_filter_args(filters={})
-        exclude_cols = self.datamodel.get_relation_filters(filters)
+        self._get_filter_args()
+        exclude_cols = self._filters.get_relation_cols()
 
         if request.method == 'POST':
             form = self.edit_form(request.form)
@@ -734,7 +799,7 @@ class GeneralView(BaseCRUDView):
                 form.populate_obj(item)
 
                 for filter_key in exclude_cols:
-                    rel_obj = self.datamodel.get_related_obj(filter_key, filters.get(filter_key))
+                    rel_obj = self.datamodel.get_related_obj(filter_key, self._filters.get_filter_value(filter_key))
                     setattr(item, filter_key, rel_obj)
                 
                 self.pre_update(item)
