@@ -29,7 +29,7 @@ class FormWidget(RenderTemplateWidget):
     exclude_cols = []
     fieldsets = []
 
-    def __init__(self, route_base='', form = None, include_cols = [], exclude_cols=[], fieldsets = []):
+    def __init__(self, route_base='', form = None, include_cols = [], exclude_cols=[], fieldsets = [], **kwargs):
         self.route_base = route_base
         self.form = form
         self.include_cols = include_cols
@@ -47,9 +47,33 @@ class FormWidget(RenderTemplateWidget):
 
 class SearchWidget(FormWidget):
     template = 'appbuilder/general/widgets/search.html'
+    filters = None
+    
     def __init__(self, **kwargs):
+        self.filters = kwargs.get('filters')
         return super(SearchWidget, self).__init__(**kwargs)
 
+    def __call__(self, **kwargs):
+        """ create dict labels based on form """
+        """ create dict of form widgets """
+        """ create dict of possible filters """
+        """ create list of active filters """
+        label_columns = {}
+        form_fields = {}
+        search_filters = {}
+        dict_filters = self.filters.get_search_filters()
+        for col in self.include_cols:
+            label_columns[col] = self.form[col].label.text.encode('utf-8')
+            form_fields[col] = self.form[col]()
+            search_filters[col] = [flt.name.encode('utf-8') for flt in dict_filters[col]]                
+
+        kwargs['label_columns'] = label_columns
+        kwargs['form_fields'] = form_fields
+        kwargs['search_filters'] = search_filters
+        kwargs['active_filters'] = self.filters.get_filters_values_tojson()
+        
+        return super(SearchWidget, self).__call__(**kwargs)
+
 class ShowWidget(RenderTemplateWidget):
 
     template = 'appbuilder/general/widgets/show.html'
